import { Option as CommanderOption, program } from 'commander';
import { satisfies as semverSatisfies } from 'semver';
import { npmPackage } from '../lib/pkgInfo.cjs';
import { commandCheck } from './commandCheck.mjs';
import { commandLink } from './commandLink.mjs';
import { commandLint } from './commandLint.mjs';
import { commandSuggestion } from './commandSuggestion.mjs';
import { commandTrace } from './commandTrace.mjs';
import { ApplicationError } from './util/errors.mjs';
export { CheckFailed } from './util/errors.mjs';
export async function run(command, argv) {
    const prog = command || program;
    const args = argv || process.argv;
    prog.exitOverride();
    prog.version(npmPackage.version).description('Spelling Checker for Code').name('cspell');
    if (!semverSatisfies(process.versions.node, npmPackage.engines.node)) {
        throw new ApplicationError(`Unsupported NodeJS version (${process.versions.node}); ${npmPackage.engines.node} is required`);
    }
    const optionFlags = new CommanderOption('-f,--flag <flag:value>', 'Declare an execution flag value')
        .hideHelp()
        .argParser((value, prev) => prev?.concat(value) || [value]);
    commandLint(prog).addOption(optionFlags);
    commandTrace(prog).addOption(optionFlags);
    commandCheck(prog).addOption(optionFlags);
    commandSuggestion(prog).addOption(optionFlags);
    commandLink(prog);
    /*
        program
            .command('init')
            .description('(Alpha) Initialize a cspell.json file.')
            .option('-o, --output <cspell.json>', 'define where to write file.')
            .option('--extends <cspell.json>', 'extend an existing cspell.json file.')
            .action((options: InitOptions) => {
                showHelp = false;
                CSpellApplication.createInit(options).then(
                    () => process.exit(0),
                    () => process.exit(1)
                );
                console.log('Init');
            });
    */
    prog.exitOverride();
    await prog.parseAsync(args);
}
